//=============================================================================
// MCoopPlayerMenu.
//
// Copyright(c) 2003, Winged Unicorn.
//=============================================================================
class MCoopPlayerMenu extends UnrealShortMenu;


struct MCoopPlayer
{
	var string PlayerClass;		// the name of the class
	var name   AnimSequence;	// the name of the animation to play
	var float  DrawScale;		// the size of the mesh
	var byte   PawnType;		// pawn type
};

var MCoopPlayer PlayerClasses[96];
var int NumPlayerClasses;

var Actor RealOwner;
var bool bSetup, bPulseDown;
var string PlayerName;
var int CurrentTeam;
var localized string Teams[4];
var string PreferredSkin;		// not a config var anymore
var string ClassString;			// not a config var anymore


// 1. Load all the pawns (loading PawnsList)
// 1. Find out the available ones (checking PawnsInfo)
// 2. Final load all of them
function LoadAvailablePawns()
{
	local PawnsInfo PawnsInfo;
	local PawnsList PawnsList;
	local MCoopPlayer NewPlayerClass;
	local int i;

	foreach AllActors(class'PawnsInfo', PawnsInfo)
		break;

	if (PawnsInfo == None)
	{
		Log("WARNING!! MCoopPlayerMenu.LoadAvailablePawns can't find PawnsInfo!");
		ExitMenu();
	}

	// load infos about all playable classes in the server
	if ((MCoopUnrealIPlayer(PlayerOwner) != None) && (MCoopUnrealIPlayer(PlayerOwner).PawnsListClass != ""))
		PawnsList = Spawn(class<PawnsList>(DynamicLoadObject(MCoopUnrealIPlayer(PlayerOwner).PawnsListClass, class'Class')));
	else
	{
		Log("WARNING!! MCoopPlayerMenu.LoadAvailablePawns couldn't spawn class '"$MCoopUnrealIPlayer(PlayerOwner).PawnsListClass$"'!");
		MCoopUnrealIPlayer(PlayerOwner).PawnsListClass = "MCoop2.PawnsList";
		ExitMenu();
	}

	// load every player class and check if it's available right now
	PawnsList.GetPlayersInfo( i, NewPlayerClass.PlayerClass,
				     NewPlayerClass.AnimSequence,
				     NewPlayerClass.DrawScale,
				     NewPlayerClass.PawnType );

	while ( (NewPlayerClass.PlayerClass != "") && (i < 200) )
	{
		if ( (NewPlayerClass.PawnType == 255) || (PawnsInfo.Pawns[NewPlayerClass.PawnType] > 0) )
		{
			PlayerClasses[NumPlayerClasses] = NewPlayerClass;
			NumPlayerClasses++;
		}

		PawnsList.GetPlayersInfo( ++i,  NewPlayerClass.PlayerClass,
						NewPlayerClass.AnimSequence,
						NewPlayerClass.DrawScale,
						NewPlayerClass.PawnType );
	}

	PawnsList.Destroy();
}

function FindSkin(int Dir)
{
	local string SkinName, SkinDesc, MeshName;
	local texture NewSkin;
	local int pos;

	// FIXME: ability to load external skins is enabled for NOW.
	// could have to be removed tho...
	MeshName = GetItemName(string(Mesh));
	SkinName = string(Skin);

	if ( Left(SkinName, Len(MeshName)) != MeshName )
		SkinName = MeshName$"Skins."$GetItemName(SkinName);
	GetNextSkin(MeshName, SkinName, Dir, SkinName, SkinDesc);
	if ( SkinName != "" )
	{
		NewSkin = Texture(DynamicLoadObject(SkinName, class'Texture'));
		if ( NewSkin == None )
			NewSkin = Texture(DynamicLoadObject(string(Skin), class'Texture'));

		Skin = NewSkin;
	}
}

function ProcessMenuInput( coerce string InputString )
{
	InputString = Left(InputString, 20);

	if ( selection == 1 )
	{
		PlayerOwner.ChangeName(InputString);
		PlayerName = PlayerOwner.PlayerReplicationInfo.PlayerName;
		PlayerOwner.UpdateURL("Name", InputString, True);
	}
}

function ProcessMenuEscape()
{
	PlayerName = PlayerOwner.PlayerReplicationInfo.PlayerName;
}

function ProcessMenuUpdate( coerce string InputString )
{
	InputString = Left(InputString, 20);

	if ( selection == 1 )
		PlayerName = (InputString$"_");
}

function Menu ExitMenu()
{
	SetOwner(RealOwner);
	Super.ExitMenu();
}

function bool ProcessLeft()
{
	if ( Selection == 1 )
	{
		PlayerName = "_";
		PlayerOwner.Player.Console.GotoState('MenuTyping');
	}
	else if ( Selection == 2 )
	{
		CurrentTeam--;
		if (CurrentTeam < 0)
			CurrentTeam = 1;
	}
	else if ( Selection == 3 )
		FindSkin(-1);

	return True;
}

function bool ProcessRight()
{
	if ( Selection == 1 )
	{
		PlayerName = "_";
		PlayerOwner.Player.Console.GotoState('MenuTyping');
	}
	else if ( Selection == 2 )
	{
		CurrentTeam++;
		if (CurrentTeam > 1)
			CurrentTeam = 0;
	}
	else if ( Selection == 3 )
		FindSkin(1);

	return True;
}

function bool ProcessSelection()
{
	if ( Selection == 1 )
	{
		PlayerName = "_";
		PlayerOwner.Player.Console.GotoState('MenuTyping');
	}

	return True;
}

function SaveConfigs()
{
	if ( ClassString == "" )
	{
		ClassString = string(PlayerOwner.Class);
		Skin = PlayerOwner.Skin;
	}

	PreferredSkin = string(Skin);
	if ( Mesh == PlayerOwner.Mesh )
		PlayerOwner.ServerChangeSkin( PreferredSkin, "", CurrentTeam );

	if ( CurrentTeam != PlayerOwner.PlayerReplicationInfo.Team )
		PlayerOwner.ChangeTeam(CurrentTeam);

	PlayerOwner.SaveConfig();
}

function MenuTick(float DeltaTime)
{
	local int I;
	
	Super.MenuTick(DeltaTime);

	// Update FadeTimes.
	if (TitleFadeTime >= 0.0)
		TitleFadeTime += DeltaTime;

	for (I=0; I<24; I++)
		if (MenuFadeTimes[I] >= 0.0)
			MenuFadeTimes[I] += DeltaTime;
}

function SetUpDisplay()
{
	local int I, PlayerClassNum;

	bSetup = True;

	// load pawns
	LoadAvailablePawns();

	// Init the FadeTimes.
	// -1.0 means not updated.
	TitleFadeTime = -1.0;
	for (I=0; I<24; I++)
		MenuFadeTimes[I] = -1.0;

	RealOwner = Owner;
	SetOwner(PlayerOwner);
	CurrentTeam = PlayerOwner.PlayerReplicationInfo.Team;
	PlayerName = PlayerOwner.PlayerReplicationInfo.PlayerName;
	PlayerOwner.bBehindView = False;

	for ( I=0; I<ArrayCount(PlayerClasses) && (PlayerClasses[I].PlayerClass != ""); I++ )
		if ( PlayerClasses[I].PlayerClass ~= string(PlayerOwner.Class) )
		{
			PlayerClassNum = I;
			break;
		}

	Mesh = PlayerOwner.Mesh;
	Skin = PlayerOwner.Skin;
	FindSkin(0);
	DrawScale = PlayerClasses[PlayerClassNum].DrawScale;

	// let me choose my animation! ;)
	if ( Mesh != None )
		LoopAnim(PlayerClasses[PlayerClassNum].AnimSequence);
}

function DrawMenu(canvas Canvas)
{
	local int i, StartX, StartY, Spacing;
	local vector DrawOffset, DrawLoc;
	local rotator NewRot, DrawRot;

	if ( !bSetup )
		SetUpDisplay();

	// Set menu location.
	if ( PlayerOwner.ViewTarget == None )
	{
		PlayerOwner.ViewRotation.Pitch = 0;
		PlayerOwner.ViewRotation.Roll = 0;
		DrawRot = PlayerOwner.ViewRotation;
		DrawOffset = ((vect(10.0,-5.0,0.0)) >> PlayerOwner.ViewRotation);
		DrawLoc = PlayerOwner.Location + PlayerOwner.EyeHeight * vect(0,0,1);
	}
	else
	{
		DrawLoc = PlayerOwner.ViewTarget.Location;
		DrawRot = PlayerOwner.ViewTarget.Rotation;
		if ( Pawn(PlayerOwner.ViewTarget) != None )
		{
			if ( (Level.NetMode == NM_StandAlone) && (PlayerOwner.ViewTarget.IsA('PlayerPawn') || PlayerOwner.ViewTarget.IsA('Bot')) )
				DrawRot = Pawn(PlayerOwner.ViewTarget).ViewRotation;

			DrawLoc.Z += Pawn(PlayerOwner.ViewTarget).EyeHeight;
		}
	}
	DrawOffset = (vect(10.0,-5.0,0.0)) >> DrawRot;
	SetLocation(DrawLoc + DrawOffset);
	NewRot = DrawRot;
	NewRot.Yaw = Rotation.Yaw;
	SetRotation(NewRot);
	Canvas.DrawActor(Self, False);

	// Draw title.
	DrawFadeTitle(Canvas);

	Spacing = Clamp(0.04 * Canvas.ClipY, 12, 32);
	StartX = Canvas.ClipX/2;
	StartY = Max(40, 0.5 * (Canvas.ClipY - MenuLength * Spacing - 128));

	for ( i=1; i<6; i++ )
		MenuList[i] = Default.MenuList[i];
	DrawFadeList(Canvas, Spacing, StartX, StartY);  

	if ( !PlayerOwner.Player.Console.IsInState('MenuTyping') )
		PlayerName = PlayerOwner.PlayerReplicationInfo.PlayerName;
	MenuList[1] = PlayerName;
	MenuList[2] = Teams[Clamp(CurrentTeam,0,3)];

	if ( Mesh != None )
		MenuList[3] = GetItemName(string(Skin));
	else
		MenuList[3] = "";

	MenuList[5] = "";
	DrawFadeList(Canvas, Spacing, StartX + 80, StartY);  

	// Draw help panel.
	Canvas.DrawColor.R = 0;
	Canvas.DrawColor.G = 255;
	Canvas.DrawColor.B = 0;
	DrawHelpPanel(Canvas, Canvas.ClipY - 64, 228);
}

defaultproperties
{
				Teams(0)="Humans"
				Teams(1)="Monsters"
				Teams(2)="Natives"
				PreferredSkin="FemaleOneSkins.Gina"
				ClassString="MCoop2.MCoopFemaleOne"
				MenuLength=3
				HelpMessage(1)="Hit enter to change your name."
				HelpMessage(2)="Use the arrow keys to change your team (Humans, Monsters)."
				HelpMessage(3)="Change your skin using the left and right arrow keys."
				MenuList(1)="Name: "
				MenuList(2)="Team:"
				MenuList(3)="Skin:"
				MenuTitle="Select Digital Representation"
				Physics=PHYS_Rotating
				AnimSequence="Walk"
				DrawType=DT_Mesh
				DrawScale=0.100000
				bUnlit=True
				bOnlyOwnerSee=True
				bFixedRotationDir=True
				RotationRate=(Yaw=8000)
				DesiredRotation=(Yaw=30000)
}
